import { Component, dragonBones, Node, sys, View } from "cc";

class FrameUtils {
    /**
    * 绑定名称以 '-' 开头的节点
    * @param {Node} node
    * @param {Component} target 
    */
    public static bindNode(node: Node, target: Component) {
        node.children.forEach((child) => {
            let name: string = child.name;
            if (name[0] === '_') {
                // 绑定节点，以节点名为key
                // @ts-ignore
                if (!target[name]) {
                    // @ts-ignore
                    target[name] = child;
                } else {
                    console.error(`${name} 节点重名，请注意`);
                }
                // 绑定节点组件， 以 $ + 组件名 为key 方便使用
                child["_components"].forEach((nodeComponent: any) => {
                    let name = FrameUtils.getComponentName(nodeComponent);
                    name = `$${name}`;
                    // @ts-ignore
                    child[name] = nodeComponent;
                });
            }
            FrameUtils.bindNode(child, target);
        });
    }
    /**
     * onNodeDestroy
     * @param node 
     * @param callback 
     */
    public static onNodeDestroy(node: Node, callback: () => void) {
        const preDestroy = node['_onPreDestroy']
        node['_onPreDestroy'] = () => {
            callback()
            preDestroy.bind(node)()
        }
    }
    /**
     * 释放node节点及所有子节点上的龙骨
     * @param node 
     */
    public static releaseNodeDragbones(node: Node) {
        if (!node) {
            return;
        }
        node.children.forEach((child) => {
            // 绑定节点组件， 以 $ + 组件名 为key 方便使用
            child["_components"].forEach((nodeComponent) => {
                let name = FrameUtils.getComponentName(nodeComponent);
                if (name === 'ArmatureDisplay') {
                    if (sys.isNative) {
                        let display: dragonBones.ArmatureDisplay | null = child.getComponent(dragonBones.ArmatureDisplay);
                        // @ts-ignore
                        display._factory.clear(false);
                        display?.destroy();
                    }
                }
            });
            FrameUtils.releaseNodeDragbones(child);
        });
    }
    /**
     * 获取组件名字
     * @param {Component} component 
     */
    private static getComponentName(component: Component | null): string {
        if (!component) {
            return '';
        }
        let nm = component.name;
        return nm.match(/<.*>$/)![0].slice(1, -1);
    }

}

const WIN_SIZE = View.instance.getVisibleSize();

export {
    FrameUtils,
    WIN_SIZE
}