import { v2, Vec2 } from "cc";

/**
 * MyMath
 */
export class MyMath {
    /**
     * 随机
     * @param min 
     * @param max 
     * @param fixed 
     */
    public static randomNum(min: number, max: number, fixed: number = 0) {
        const r = Math.random() * (max - min + 1) + min
        if (fixed <= 0) {
            return parseInt(String(r))
        } else {
            return +r.toFixed(fixed)
        }
    }

    /**
     * 计算两点之间距离
     * @param p0
     * @param p1
     */
    public static pointDistance(p0: Vec2, p1: Vec2): number {
        let self = this;
        let dis: number = 0;
        let x2 = (p0.x - p1.x) * (p0.x - p1.x);
        let y2 = (p0.y - p1.y) * (p0.y - p1.y);
        dis = Math.sqrt(x2 + y2);
        return dis;
    }

    /**
     * Calculate length of bezier curve.
     */
    public static bezierLength2(
        fromX: number, fromY: number, cpX: number, cpY: number, toX: number,
        toY: number): number {
        const n = 10;
        let result = 0.0;
        let t = 0.0;
        let t2 = 0.0;
        let t3 = 0.0;
        let nt = 0.0;
        let nt2 = 0.0;
        let nt3 = 0.0;
        let x = 0.0;
        let y = 0.0;
        let dx = 0.0;
        let dy = 0.0;
        let prevX = fromX;
        let prevY = fromY;
        let d = 2.0;
        for (let i = 1; i <= n; ++i) {
            t = i / n;
            t2 = t;
            t3 = t2 * t;
            nt = (1.0 - t);
            nt2 = nt;
            nt3 = nt2 * nt;
            x = (nt3 * fromX) + (d * nt2 * t * cpX) + (t3 * toX);
            y = (nt3 * fromY) + (d * nt2 * t * cpY) + (t3 * toY);
            dx = prevX - x;
            dy = prevY - y;
            prevX = x;
            prevY = y;
            result += Math.sqrt((dx * dx) + (dy * dy));
        }
        return result;
    }

    public static bezierLength3(
        fromX: number, fromY: number, cpX: number, cpY: number, cpX2: number,
        cpY2: number, toX: number, toY: number): number {
        const n = 10;
        let result = 0.0;
        let t = 0.0;
        let t2 = 0.0;
        let t3 = 0.0;
        let nt = 0.0;
        let nt2 = 0.0;
        let nt3 = 0.0;
        let x = 0.0;
        let y = 0.0;
        let dx = 0.0;
        let dy = 0.0;
        let prevX = fromX;
        let prevY = fromY;
        let d = 3.0;
        for (let i = 1; i <= n; ++i) {
            t = i / n;
            t2 = t * t;
            t3 = t2 * t;
            nt = (1.0 - t);
            nt2 = nt * nt;
            nt3 = nt2 * nt;
            x = (nt3 * fromX) + (d * nt2 * t * cpX) + (d * nt * t2 * cpX2) +
                (t3 * toX);
            y = (nt3 * fromY) + (d * nt2 * t * cpY) + (d * nt * t2 * cpY2) +
                (t3 * toY);
            dx = prevX - x;
            dy = prevY - y;
            prevX = x;
            prevY = y;
            result += Math.sqrt((dx * dx) + (dy * dy));
        }
        return result;
    }

    /**
     * Calculate the points for a bezier curve and then draws it.
     */
    public static bezier2To(
        fromX: number, fromY: number, cpX: number, cpY: number, toX: number,
        toY: number): Array<Vec2> {
        let points: Array<Vec2> = [];
        const n = MyMath.bezierLength2(fromX, fromY, cpX, cpY, toX, toY);
        let dt = 0;
        let dt2 = 0;
        let dt3 = 0;
        let t2 = 0;
        let t3 = 0;
        points[points.length] = v2(fromX, fromY);
        let d = 2;
        for (let i = 1, j = 0; i <= n; ++i) {
            j = i / n;
            dt = (1 - j);
            dt2 = dt;
            dt3 = dt2 * dt;
            t2 = j;
            t3 = t2 * j;
            points[points.length] = v2(
                (dt3 * fromX) + (d * dt2 * j * cpX) + (t3 * toX),
                (dt3 * fromY) + (d * dt2 * j * cpY) + (t3 * toY));
        }
        return points;
    }

    public static bezier3To(
        fromX: number, fromY: number, cpX: number, cpY: number, cpX2: number,
        cpY2: number, toX: number, toY: number): Array<Vec2> {
        let points: Array<Vec2> = [];
        const n =
            MyMath.bezierLength3(fromX, fromY, cpX, cpY, cpX2, cpY2, toX, toY);
        let dt = 0;
        let dt2 = 0;
        let dt3 = 0;
        let t2 = 0;
        let t3 = 0;
        points[points.length] = v2(fromX, fromY);
        let d = 3;
        for (let i = 1, j = 0; i <= n; ++i) {
            j = i / n;
            dt = (1 - j);
            dt2 = dt * dt;
            dt3 = dt2 * dt;
            t2 = j * j;
            t3 = t2 * j;
            points[points.length] = v2(
                (dt3 * fromX) + (d * dt2 * j * cpX) + (d * dt * t2 * cpX2) +
                (t3 * toX),
                (dt3 * fromY) + (d * dt2 * j * cpY) + (d * dt * t2 * cpY2) +
                (t3 * toY));
        }
        return points;
    }

    public static rand(rMi: number, rMa: number) {
        return ~~((Math.random() * (rMa - rMi + 1)) + rMi);
    }


    /**
     *  向量
     */

    /**
     * 计算两个向量的夹角
     * @param v1
     * @param v2
     */
    public static getAngle(v1: Vec2, v2: Vec2): number {
        if (typeof v1 !== 'object' || typeof v2 !== 'object') {
            console.error('getAngle error!');
            return 0;
        }
        // 判断方向，顺时针为 1 ,逆时针为 -1；
        let direction = v1.x * v2.y - v2.x * v1.y > 0 ? 1 : -1;
        // 两个向量的模；
        let len1 = MyMath.getLength(v1);
        let len2 = MyMath.getLength(v2);
        let mr = len1 * len2;
        let dot;
        let r;
        if (mr === 0) {
            return 0;
        }
        // 通过数量积公式可以推导出：
        // cos = (x1 * x2 + y1 * y2)/(|a| * |b|);
        dot = v1.x * v2.x + v1.y * v2.y;
        r = dot / mr;
        if (r > 1) r = 1;
        if (r < -1) r = -1;
        // 解值并结合方向转化为角度值；
        return Math.acos(r) * direction * 180 / Math.PI;
    }

    /**
     * 计算向量
     * @param p1 终点
     * @param p2 起点
     */
    public static getVector(p1: Vec2, p2: Vec2) {
        if (typeof p1 !== 'object' || typeof p2 !== 'object') {
            console.error('getvector error!');
            return;
        }
        let x = Math.round(p1.x - p2.x);
        let y = Math.round(p1.y - p2.y);
        return { x, y };
    }

    public static getLength(v1: Vec2): number {
        if (typeof v1 !== 'object') {
            console.error('getLength error!');
            return 0;
        }
        return Math.sqrt(v1.x * v1.x + v1.y * v1.y);
    }

    public static dotVec(v1: Vec2, v2: Vec2) {
        let l1 = MyMath.getLength(v1);
        let l2 = MyMath.getLength(v2);
        let a = MyMath.getAngle(v1, v2);
        let ar = Math.PI * a / 180;
        let result = l1 * l2 * Math.cos(ar);
        return result;
    }
}