/**
 * 定时器
 */
class MyTimer {
    private static _intervalIDArray: Array<number> = [];   // 立即执行定时器ID容器

    /**
     * 创建定时器并返回ID
     * @param callback 
     * @param interval 间隔、延时，单位ms，0立刻执行
     * @param repeat 重复次数，-1无限循环，默认1次不需传入，0会自动转成1
     * @param delay 延时执行，ms    // 参数废弃
     */
    public static schedule(callback: Function, interval?: number, repeat?: number, delay?: number): number {
        let runTime = 0;
        let inter = interval || 1000;   // repeat time interval
        if (interval === 0) {
            inter = 0;
        }
        let rep = repeat || 1;  // repeat times
        let del = delay || 0;   // delay time
        let CB = () => {
            callback();
            if (rep > 0 && ++runTime >= rep) {
                MyTimer.unschedule(ID);
            }
        }
        let ID = setInterval(CB, inter);
        MyTimer._intervalIDArray[MyTimer._intervalIDArray.length] = ID;
        return ID;
    }

    /**
     * 停止定时器
     * @param ID 
     */
    public static unschedule(ID: number) {
        clearInterval(ID);
        for (let i = 0; i < MyTimer._intervalIDArray.length; i++) {
            if (ID === MyTimer._intervalIDArray[i]) {
                MyTimer._intervalIDArray.splice(i, 1);
                break;
            }
        }
    }

    /**
     * 停止全部定时器
     */
    public static unscheduleAll() {
        for (let i = 0; i < MyTimer._intervalIDArray.length; i++) {
            let ID = MyTimer._intervalIDArray[i];
            clearInterval(ID);
        }
        MyTimer._intervalIDArray.length = 0;
    }
}

export {
    MyTimer
}