import { v2, Vec2 } from "cc";

class MapData {
    /**
     * 单例
     */
    private static singleInstance: MapData | null = null;
    static getInstance(): MapData {
        if (MapData.singleInstance == null) {
            MapData.singleInstance = new MapData();
        }
        return MapData.singleInstance;
    }

    public static delInsTan() {
        MapData.singleInstance = null;
    }

    public stoneID: number = -1;
    public grassID: number = -1;

    public mapsStone: Array<Array<number>> | null = null;
    public mapsGrass: Array<Array<number>> | null = null;

    /**
     * 玩家出生地点数组
     */
    private validBirthGrid: Array<number> | null = null;

    constructor() {
        this.mapsStone = [];
        this.mapsGrass = [];
        this.validBirthGrid = [];
        this.resetData();
    }

    public resetData() {
        this.mapsStone!.length = 0;
        for (let i = 0; i < MapGridY; i++) {
            this.mapsStone![i] = [];
            for (let j = 0; j < MapGridX; j++) {
                this.mapsStone![i][j] = 0;
            }
        }
        this.mapsGrass!.length = 0;
        for (let i = 0; i < MapGridY; i++) {
            this.mapsGrass![i] = [];
            for (let j = 0; j < MapGridX; j++) {
                this.mapsGrass![i][j] = 0;
            }
        }
        this.validBirthGrid!.length = 0;
        for (let i = 0; i < MapGridX * MapGridY; i++) {
            this.validBirthGrid![i] = i;
        }
    }

    private removeGrid(grid: number) {
        for (let i = 0; i < this.validBirthGrid!.length; ++i) {
            if (this.validBirthGrid![i] === grid) {
                this.validBirthGrid!.splice(i, 1);
                break;
            }
        }
    }

    public updateStoneData(tiles: Array<number>) {
        for (let i = 0; i < tiles.length; i++) {
            let y = Math.floor(i / MapGridY);
            let x = i % MapGridX;
            this.mapsStone![x][y] = tiles[i];
            //
            if (tiles[i] !== 0) {
                this.removeGrid(i);
            }
        }
    }
    public updateGrassData(tiles: Array<number>) {
        for (let i = 0; i < tiles.length; i++) {
            let y = Math.floor(i / MapGridY);
            let x = i % MapGridX;
            this.mapsGrass![x][y] = tiles[i];
            //
            if (tiles[i] !== 0) {
                this.removeGrid(i);
            }
        }
    }

    private validX(x: number): boolean {
        if (x < 0 || x >= MapGridX) {
            return false;
        }
        return true;
    }
    private validY(y: number): boolean {
        if (y < 0 || y >= MapGridY) {
            return false;
        }
        return true;
    }

    public isGridStone(x: number, y: number): boolean {
        if (!this.validX(x) || !this.validY(y)) {
            return false;
        }
        return this.mapsStone![x][y] > 0;
    }
    public isGridGrass(x: number, y: number): boolean {
        if (!this.validX(x) || !this.validY(y)) {
            return false;
        }
        return this.mapsGrass![x][y] > 0;
    }

    /**
     * 周围最多8个格子
     * @param x 
     * @param y 
     */
    public nearGrid(x: number, y: number): Array<Vec2> {
        if (!this.validX(x) || !this.validY(y)) {
            return [];
        }
        let l = 1;
        let result = [];
        for (let i = x - l; i <= x + l; ++i) {
            for (let j = y - l; j <= y + l; ++j) {
                if (this.validX(i) && this.validY(j) && !(i === x && j === y)) {
                    result[result.length] = v2(i, j);
                }
            }
        }
        return result;
    }

    /**
     * 行进方向可能产生碰撞的格子
     * @param x 
     * @param y 
     * @param dx 
     * @param dy 
     */
    public nearTargetGrid(x: number, y: number, dx: number, dy: number): Array<Vec2> {
        if (!this.validX(x) || !this.validY(y)) {
            return [];
        }
        if (dx === 0 && dy === 0) {
            return [];
        }
        let lx = dx < 0 ? 1 : 0;
        let rx = dx > 0 ? 1 : 0;
        let ly = dy > 0 ? 1 : 0;
        let ry = dy < 0 ? 1 : 0;
        let result = [];
        for (let i = x - lx; i <= x + rx; ++i) {
            for (let j = y - ly; j <= y + ry; ++j) {
                if (this.validX(i) && this.validY(j) && !(i === x && j === y)) {
                    result[result.length] = v2(i, j);
                }
            }
        }
        return result;
    }

    public gridCenterPos(gx: number, gy: number): Vec2 {
        let x = (gx + 0.5) * MapTileWidth;
        let y = 0 - (gy + 0.5) * MapTileHeight;
        return v2(x, y);
    }
    public gridTLPos(gx: number, gy: number): Vec2 {
        let x = gx * MapTileWidth;
        let y = 0 - gy * MapTileHeight;
        return v2(x, y);
    }
    public gridBLPos(gx: number, gy: number): Vec2 {
        let x = gx * MapTileWidth;
        let y = 0 - (gy + 1) * MapTileHeight;
        return v2(x, y);
    }

    /**
     * 是否在地图范围内
     * @param x 
     * @param y 
     * @param node 
     */
    public inMap(x: number, y: number, node: Node): boolean {
        let inX = this.inMapX(x, node);
        let inY = this.inMapY(y, node);
        return inX && inY;
    }
    private inMapX(x: number, node: Node): boolean {
        // let hw = node.width / 2;
        // let width = MapGridX * MapTileWidth;
        // if (x >= hw && x <= width - hw) {
        //     return true;
        // }
        return false;
    }
    private inMapY(y: number, node: Node): boolean {
        // let hh = node.height / 2;
        // let height = MapGridY * MapTileHeight;
        // y = 0 - y;
        // if (y >= hh && y <= height - hh) {
        //     return true;
        // }
        return false
    }

    /**
     * 修正x坐标
     * @param dx 
     * @param node 
     */
    public fixMapX(dx: number, node: Node): number {
        // let x = node.x + dx;
        // if (this.inMapX(x, node)) {
        //     return dx;
        // }
        // let hw = node.width / 2;
        // let width = MapGridX * MapTileWidth;
        // if (x < hw) {
        //     dx = hw - node.x;
        // }
        // if (x > width - hw) {
        //     dx = width - hw - node.x;
        // }
        return dx;
    }
    /**
     * 修正y坐标
     * @param dy 
     * @param node 
     */
    public fixMapY(dy: number, node: Node): number {
        // let y = node.y + dy;
        // if (this.inMapY(y, node)) {
        //     return dy;
        // }
        // let hh = node.height / 2;
        // let height = MapGridY * MapTileHeight;
        // if (y > 0 - hh) {
        //     dy = 0 - hh - node.y;
        // }
        // if (y < 0 - (height - hh)) {
        //     dy = 0 - (height - hh) - node.y;
        // }
        return dy;
    }

    /**
     * 出生点有效数组
     */
    public validBirthGridLen(): number {
        return this.validBirthGrid!.length;
    }
    public getValidBirthGrid(idx: number): number {
        let grid: number = this.validBirthGrid!.splice(idx)[0];
        return grid;
    }
    public getPlayerBirthPos(grid: number): Vec2 {
        let y = Math.floor(grid / MapGridY);
        let x = grid % MapGridX;
        let result = v2((x + 0.5) * MapTileWidth, 0 - (y + 0.5) * MapTileHeight);
        return result;
    }

}

class TileSetsName {
    public static stone: string = 'stone'
    public static grass: string = 'grass'
}

const MapGridX: number = 16;
const MapGridY: number = 16;

const MapTileWidth: number = 60;
const MapTileHeight: number = 60;

const PlayerWd: number = 40;


export {
    MapData,
    TileSetsName,
    MapGridX,
    MapGridY,
    MapTileWidth,
    MapTileHeight,
    PlayerWd
}