
import { _decorator, Component, Node, Vec2, v2, EventTouch, UI, UITransform, Sprite, v3, Vec3, SystemEventType, math } from 'cc';
import { FrameUtils } from '../../frame/FrameUtils';
import { MyMath } from '../../frame/MyMath';
import { BattleData } from '../data/BattleData';
import { FrameSyncCmd } from '../logic/FrameSyncLogic';

const max_R: number = 130;

const { ccclass, property } = _decorator;

@ccclass('JoyStick')
export class JoyStick extends Component {
    /**
     * 是否正在生效
     * 生效时自己角色上报帧更新数据受影响
     */
    public isWorking: boolean = false;

    public _bg: Node | null = null;
    public _joystick: Node | null = null;

    private rotation: number = 0;
    private vector: Vec2 = v2(0, 0);
    private is_rotation: boolean = true;
    private is_forbidden: boolean = true;

    onLoad() {
        FrameUtils.bindNode(this.node, this);
        this._bg?.on(SystemEventType.TOUCH_START, this._touchStart, this);
        this._bg?.on(SystemEventType.TOUCH_MOVE, this._move, this);
        this._bg?.on(SystemEventType.TOUCH_END, this._finish, this);
        this._bg?.on(SystemEventType.TOUCH_CANCEL, this._finish, this);
    }

    start() {
        this.forbidJoyStick();
        this._finish();

        this.activeJoyStick();
    }

    update() {
        if (!this.isWorking) {
            return;
        }
    }

    public activeJoyStick() {
        this.is_forbidden = false;
    }
    public forbidJoyStick() {
        this.is_forbidden = true;
    }

    private _touchStart(event: EventTouch) {
        if (this.is_forbidden === false) {
            this._updateJoyStickAlpha(255);
            this._updateJoyStick(event);
        }
    }

    private _move(event: EventTouch) {
        if (this.is_forbidden == false) {
            this.isWorking = true;
            this._updateJoyStick(event);
        }
    }

    private _updateJoyStick(event: EventTouch) {
        let uiLocation = event.getUILocation();
        let pos = v3(uiLocation.x, uiLocation.y, 0);
        let uiTrans = this.node?.getComponent(UITransform);
        let pos_0: Vec3 | undefined = uiTrans?.convertToNodeSpaceAR(pos);
        let len = pos_0?.length();
        let dir = v2(pos_0?.x, pos_0?.y);
        dir = dir.normalize();
        if (len && len < max_R) {
            pos_0 && this._joystick?.setPosition(pos_0);
        } else {
            let pos = pos_0?.normalize();
            let x = pos!.x * max_R;
            let y = pos!.y * max_R;
            this._joystick?.setPosition(v3(x, y, 0));
            dir = v2(pos?.x, pos?.y);
        }
        // 
        if (this.is_forbidden == false) {
            let angle: number = MyMath.getAngle(v2(1, 0), v2(pos_0?.x, pos_0?.y));
            this._move_player(dir, angle);
        }
    }

    private _move_player(dir: Vec2, angle: number): void {
        let B_DATA = BattleData.getInstance();

        if (this.is_forbidden == false) {
            let myPlayer = B_DATA.myPlayer();
            if (!myPlayer) {
                console.warn('my player is nil');
                return;
            }
            let rotation = angle / 180 * Math.PI;

            myPlayer.frameCmd = FrameSyncCmd.walk;
            myPlayer.vector.x = dir.x;
            myPlayer.vector.y = dir.y;
            myPlayer.stickRotation = rotation;
        }
    }

    private _finish() {
        this._joystick?.setPosition(v3(0, 0, 0));
        this.vector = v2(0, 0);
        this._updateJoyStickAlpha(100);
        this.isWorking = false;
        //
        let B_DATA = BattleData.getInstance();
        let myPlayer = B_DATA.myPlayer();
        if (myPlayer) {
            myPlayer.frameCmd = FrameSyncCmd.stop;
            myPlayer.stickRotation = 0;
            myPlayer.vector.x = 0;
            myPlayer.vector.y = 0;
        }
    }

    private _updateJoyStickAlpha(opacity: number): void {
        this._bg?.getComponent(Sprite)?.color.set(255, 255, 255, opacity);
        this._joystick?.getComponent(Sprite)?.color.set(255, 255, 255, opacity);
    }
}
