
import { _decorator, Component, Node, Vec2, v2, UITransform, SkeletalAnimation, Camera, v3, Quat } from 'cc';
import { FrameUtils } from '../../frame/FrameUtils';
import { FollowCameraPos } from '../data/BattleData';
import { MapData, MapTileHeight, MapTileWidth, PlayerWd } from '../data/MapData';
import { RoomData } from '../data/RoomData';
import { FrameSyncCmd } from '../logic/FrameSyncLogic';
import { MapCtrl } from '../map/MapCtrl';
const { ccclass, property } = _decorator;

@ccclass('PlayerBase')
export class PlayerBase extends Component {

    public mainCamera: Node | null = null;

    public _role: Node | null = null;

    /** ID */
    public id: string | null = null;

    /** 向后端传送的指令的原始数据 */
    public vector: Vec2 = v2(0, 0);
    public stickRotation: number = 0;



    /** 旋转四元素 */
    private _rotationQuat: Quat = new Quat();




    public MAP: MapCtrl | null = null;

    public isRobot: boolean = false;

    private _speed: number = 0.1;
    private _isMe: boolean = false;



    /** 自己角色指令状态 */
    public frameCmd: FrameSyncCmd = FrameSyncCmd.stop;

    /**
     * 炸弹冷却相关
     */
    private _endFrozenFrame: number = 0;
    private _isFrozen: boolean = false;
    private _hasBomb: boolean = false;

    private _cmdCaches: Array<object> = [];

    public isFrameSync: boolean = false;

    onLoad() {
        FrameUtils.bindNode(this.node, this);
        //
        this._initData();
    }

    start() {
        this._pLayWalKani();
    }

    update(dt: number) {
        this.checkInGrass();

        // if (!this.isFrameSync) {
        //     return;
        // }

        if (this.isMe && this.frameCmd !== FrameSyncCmd.stop) {
            let cmd: any = {};
            let data: any = {};
            data.cmd = this.frameCmd;
            data.rotation = this.stickRotation;
            data.dx = this.vector.x;
            data.dz = this.vector.y;
            cmd.data = {};
            cmd.data.data = data;

            this.cmdPlayer(cmd);
        }
    }



    public sendFrameData() {
        // let R_DATA = RoomData.getInstance();
        // let data: object = {};
        // data['cmd'] = this.frameCmd;
        // data['angle'] = this.stickRotation;
        // data['dx'] = this.vector.x;
        // data['dy'] = this.vector.y;
        // data['time'] = Date.now();
        // data['bomb'] = this.isBomb();
        // const sendFramePara: MGOBE.types.SendFramePara = {
        //     data: {
        //         data,
        //     },
        // };
        // R_DATA.room.sendFrame(sendFramePara, event => {
        //     if (event.code === MGOBE.ErrCode.EC_OK) {
        //     } else {
        //         console.log(`发送帧消息失败，错误码：${event.code}`);
        //     }
        // });
    }

    /**
     * 根据后端逻辑帧刷新
     * @param id 
     */
    public svrFramUpt(id: number) {
        /**
         * 同步更新炸弹状态
         */
        this._updateFrozen(id);
    }

    private _initData() {
        this.setBomb(false);
    }

    /**
     * 
     * @param info 
     * commonNetworkState
     * customPlayerStatus
     * customProfile
     * id
     * isRobot
     * matchAttributes
     * name
     * relayNetworkState
     * teamId
     */
    public syncPlayerInfo(info: object) {
        if (!info) {
            return;
        }
        // this.id = info['id'];
        // console.log('player id: ', this.id);

        // let name = info['name'] || '';
        // this._playerName['$Label'].string = name;
        // this._isMe = info['id'] == MGOBE.Player.id;
        // this.isRobot = info['isRobot'];

        // if (this._isMe) {
        //     this._role.color = Color.RED;
        // }
    }


    /**
     * 根据帧更新指令更新角色信息
     * @param cmd 
     */
    public cmdPlayer(cmd: object) {
        this._cmdCaches[this._cmdCaches.length] = cmd;
        this.fireCmd();
    }

    private fireCmd() {
        // let MAP_DATA = MapData.getInstance();
        // if (!this.isFrameSync) {
        //     return;
        // }

        let cmdObj: any = this._cmdCaches.shift();
        if (!cmdObj) {
            return;
        }

        // 帧同步收到的帧ID
        let data = cmdObj['data'].data || {};
        let cmd = data.cmd;

        if (cmd == FrameSyncCmd.stop) {
            return;
        }


        let rotation = data.rotation;
        let dx = data.dx * this.speed;
        let dz = data.dz * this.speed;
        // dx = MAP_DATA.fixMapX(dx, this.node);
        // dy = MAP_DATA.fixMapY(dy, this.node);



        // let r = this.checkNearStone(dx, dy);
        // dx = r.x;
        // dy = r.y;




        let pos = this.node.getPosition();
        pos.x += dx;
        pos.z -= dz;

        // let x = this.node.x + dx;
        // let y = this.node.y + dy;//每帧获取角色的坐标加上移动向量
        this.roTateRole(rotation);
        // this.node.position = v3(x, y); // 让角色的坐标每帧为自身的坐标加上移动的向量
        this.node.setPosition(pos);

        /** 更新摄像机 */
        this._updateMainCamera(dx, dz);
    }

    private _updateMainCamera(dx: number, dz: number) {
        if (this.mainCamera) {
            let pos = this.mainCamera.getPosition();
            pos.x += dx;
            pos.y += dz;
            console.log('pos: ', pos.x, pos.y, pos.z);

            this.mainCamera.setPosition(pos)
        }
    }

    public roTateRole(rotation: number) {
        this._role?.getRotation(this._rotationQuat);
        Quat.rotateY(this._rotationQuat, this._rotationQuat, rotation);
        this._role?.setRotation(this._rotationQuat);
    }

    private checkNearStone(dx: number, dy: number): Vec2 {
        // let grid = this.getGrid(dx, dy);
        // let MAP_DATA = MapData.getInstance();
        // let hw = PlayerWd / 2;
        // let nears = MAP_DATA.nearTargetGrid(grid.x, grid.y, dx, dy);
        // for (let i = 0; i < nears.length; ++i) {
        //     let g = nears[i];
        //     if (MAP_DATA.isGridStone(g.x, g.y)) {
        //         let stonePos = MAP_DATA.gridBLPos(g.x, g.y);
        //         let ir: IRect = {
        //             x: stonePos.x,
        //             y: stonePos.y,
        //             width: MapTileWidth,
        //             height: MapTileHeight
        //         }
        //         let irx: IRect = {
        //             x: this.node.x - hw + dx,
        //             y: this.node.y - hw,
        //             width: PlayerWd,
        //             height: PlayerWd
        //         }
        //         let iry: IRect = {
        //             x: this.node.x - hw,
        //             y: this.node.y - hw + dy,
        //             width: PlayerWd,
        //             height: PlayerWd
        //         }
        //         if (hitTest.Rectangle(ir, irx)) {
        //             let l = stonePos.x + MapTileWidth / 2 + (MapTileWidth / 2 + hw) * (dx > 0 ? -1 : 1);
        //             dx = l - this.node.x;
        //         }
        //         if (hitTest.Rectangle(ir, iry)) {
        //             let l = stonePos.y + MapTileHeight / 2 + (MapTileHeight / 2 + hw) * (dy < 0 ? 1 : -1);
        //             dy = l - this.node.y;
        //         }
        //     }
        //     else {
        //         continue;
        //     }
        // }
        return v2(dx, dy);
    }

    private iNGrass(): boolean {
        let MAP_DATA = MapData.getInstance();
        let grid = this.getGrid();
        let iNG = MAP_DATA.isGridGrass(grid.x, grid.y);
        return iNG;
    }
    private checkInGrass() {
        // if (this.iNGrass()) {
        //     this.node.parent = this.MAP._playerBtm;
        //     if (this.isMe) {
        //         this.node.opacity = 150;
        //     }
        //     else {
        //         this.node.opacity = 0;
        //     }
        // }
        // else {
        //     this.node.parent = this.MAP._playerTop;
        //     this.node.opacity = 255;
        // }
    }

    public get isMe() {
        return true;
        return this._isMe;
    }

    public get speed() {
        return this._speed;
    }

    // 角色如果死亡，则对局里不显示
    public get isDie() {
        return false;
    }

    public getGrid(dx: number = 0, dy: number = 0): Vec2 {
        let pos = this.node.position;
        let x = Math.floor((pos.x + dx) / MapTileWidth);
        let y = Math.floor((0 - (pos.y + dy)) / MapTileHeight);
        return v2(x, y);
    }

    /**
     * 角色炸弹时间，不同角色或者不同属性该值可能会不同
     * (炸弹传递出去后这个时间内不能再次被炸弹传递)
     */
    public get bombFrozenFrame() {
        return 20;
    }

    public isBomb() {
        return this._hasBomb;
    }
    public setBomb(has: boolean, id?: number) {
        // if (this.isFrozen()) {
        //     return;
        // }
        // if (this._hasBomb && !has) {
        //     // 由带炸弹变成没有炸弹
        //     this._endFrozenFrame = id! + this.bombFrozenFrame;
        //     this._updateFrozen(id!);
        // }
        // this._hasBomb = has;
        // this._bomb && (this._bomb.active = has);
    }

    /**
     * 安全状态无法再次被传递炸弹
     */
    public isFrozen(): boolean {
        return this._isFrozen;
    }
    private _updateFrozen(id: number) {
        this._isFrozen = id < this._endFrozenFrame;
    }

    // public rect(): IRect {
    //     let hwd = PlayerWd / 2;
    //     let pos = this.node.position;
    //     const iRect: IRect = {
    //         x: pos.x - hwd,
    //         y: pos.y - hwd,
    //         width: PlayerWd,
    //         height: PlayerWd
    //     }
    //     return iRect;
    // }

    private _pLayWalKani() {
        // let RootNode = this._role?.getChildByName('RootNode');
        // let ani = RootNode?.getComponent(SkeletalAnimation);
        // ani?.play('Armature|walk');
    }

    public iSVisible(): boolean {
        if (this.iNGrass()) {
            if (!this.isMe) {
                return false;
            }
        }
        return true;
    }
}
