
import { _decorator, Component, Node, instantiate, Prefab, v3, Tween, tween, Vec3 } from 'cc';
import { FrameUtils } from '../../frame/FrameUtils';
import { ResourceManager } from '../../frame/ResourceManager';
import { BattleData, CameraTweenDuration, FollowCameraPos, WholeCameraPos } from '../data/BattleData';
import { PlayerMaxCNT, RoomData } from '../data/RoomData';
import { CustomEventListener } from '../event/CustomEventListener';
import { EventName } from '../event/EventName';
import { MapCtrl } from '../map/MapCtrl';
import { PlayerBase } from '../player/PlayerBase';
const { ccclass, property } = _decorator;

@ccclass('BattleScene')
export class BattleScene extends Component {

    public _MainCamera: Node | null = null;

    public _mapRoot: Node | null = null;

    private _mapPrefab: Prefab | null = null;
    private _mapNode: Node | null = null;
    private _mapCtrl: MapCtrl | null = null;

    private _playerPrefab: Prefab | null = null;

    /** 摄像机缓动动画 */
    private _cameraTween: Tween<any> | null = null;

    onLoad() {
        FrameUtils.bindNode(this.node, this);
        //
        this._initListener();
    }

    start() {
    }

    private _initListener() {
        CustomEventListener.on(EventName.ShowStartUI, () => {
            this._ctrlBattleView(false);
        });
        CustomEventListener.on(EventName.ShowMatchingUI, () => {
            this._ctrlBattleView(false);
        });
        CustomEventListener.on(EventName.ShowBattleUI, () => {
            let res = ResourceManager.getInstance();
            let mapPrefab = res.loadPrefab('prefabs/map/map0');
            let playerPrefab = res.loadPrefab('prefabs/role/Police');
            Promise.all([mapPrefab, playerPrefab]).then((results) => {
                this._mapPrefab = results[0];
                this._playerPrefab = results[1];
                //
                this._ctrlBattleView(true);
                this._initMap();
                this._initPlayers();
                //
                this.trans2FollowCamera(null);
            });
        });
    }

    private _initMap() {
        if (this._mapPrefab && !this._mapNode) {
            this._mapNode = instantiate(this._mapPrefab);
            this._mapNode.parent = this._mapRoot;
            this._mapCtrl = this._mapNode.addComponent(MapCtrl);
        }
    }

    private _initPlayers() {
        if (this._playerPrefab && this._mapCtrl) {
            let ids = RoomData.getInstance().getPlayersIDs();
            if (!ids) {
                return;
            }
            for (let i = 0; i < PlayerMaxCNT; ++i) {
                if (i <= ids?.length) {
                    let player = instantiate(this._playerPrefab);
                    this._mapCtrl.addPlayer(player, v3(0, 0, 0));
                    let p = player.addComponent(PlayerBase);
                    BattleData.getInstance().insertPlayer(p);

                    if (p && p.isMe) {
                        p.mainCamera = this._MainCamera;
                    }
                }
            }
        }
    }

    private _ctrlBattleView(show: boolean): void {
        this.node.active = show;
        this._mapRoot && (this._mapRoot.active = show);
    }

    /** 缓动动画变成跟随摄像机 */
    public trans2FollowCamera(CB: Function | null): void {
        if (this._cameraTween) {
            this._cameraTween.stop();
        }
        this._cameraTween = tween(this._MainCamera)
            .to(CameraTweenDuration, { position: FollowCameraPos }, { "easing": 'linear' })
            .call(() => {
                if (CB) {
                    CB();
                }
            });
        this._cameraTween.start();
    }
    /** 缓动动画变成全局摄像机 */
    public trans2WholeCamera(CB: Function | null): void {
        if (this._cameraTween) {
            this._cameraTween.stop();
        }
        this._cameraTween = tween(this._MainCamera)
            .to(CameraTweenDuration, WholeCameraPos, { "easing": 'linear' })
            .call(() => {
                if (CB) {
                    CB();
                }
            });
        this._cameraTween.start();
    }

}
